import torch
import torch.nn as nn
import torch.optim as optim
from torchvision import datasets, transforms
from torch.utils.data import DataLoader
import matplotlib.pyplot as plt

# 数据预处理
transform = transforms.Compose([
    transforms.ToTensor(),  # 转换为Tensor
    transforms.Normalize((0.5,), (0.5,))  # 归一化
])

# 加载数据集
train_dataset = datasets.MNIST(root='./data', train=True, download=True, transform=transform)
test_dataset = datasets.MNIST(root='./data', train=False, download=True, transform=transform)

# 数据加载器
# train_loader = DataLoader(dataset=train_dataset, batch_size=64, shuffle=True)
# test_loader = DataLoader(dataset=test_dataset, batch_size=64, shuffle=False)


# 调整数据形状以匹配RNN输入 (seq_len, batch, input_size)
# 包装数据加载器以改变数据形状
# 包装数据加载器以改变数据形状
def collate_fn(batch):
    # 分离数据和标签
    images, labels = zip(*batch)
    # 重塑图像数据以匹配RNN输入
    images = torch.stack([img.view(28, 28) for img in images])
    images = images.permute(1, 0, 2)  # [seq_len, batch_size, input_size]
    return images, torch.tensor(labels)


# 使用修正后的collate_fn
train_loader = DataLoader(train_dataset, batch_size=28, shuffle=True, collate_fn=collate_fn)
test_loader = DataLoader(test_dataset, batch_size=28, shuffle=False, collate_fn=collate_fn)

# def collate_fn(batch):
#     # 分离数据和标签
#     images, labels = zip(*batch)
#     # 重塑图像数据以匹配RNN输入
#     images = torch.stack([img.view(28, 28) for img in images])
#     images = images.permute(1, 0, 2)  # [seq_len, batch_size, input_size]
#     return images, torch.tensor(labels)
#
#
# # 使用修正后的collate_fn
# train_loader = DataLoader(train_dataset, batch_size=28, shuffle=True, collate_fn=collate_fn)
# test_loader = DataLoader(test_dataset, batch_size=28, shuffle=False, collate_fn=collate_fn)


class RNN(nn.Module):
    def __init__(self, input_size, hidden_size, output_size, num_layers=1):
        super(RNN, self).__init__()
        self.hidden_size = hidden_size
        self.num_layers = num_layers
        self.rnn = nn.RNN(input_size, hidden_size, num_layers, batch_first=True)
        self.fc = nn.Linear(hidden_size, output_size)

    def forward(self, x):
        # 初始化隐藏状态
        h0 = torch.zeros(self.num_layers, x.size(0), self.hidden_size).to(x.device)

        # 前向传播RNN
        out, _ = self.rnn(x, h0)

        # 取序列的最后一个输出
        out = self.fc(out[:, -1, :])
        return out

    # 模型参数

def train():
    input_size = 28  # 每个时间步的输入特征数
    hidden_size = 64  # 隐藏层特征数
    output_size = 10  # 输出层特征数（类别数）
    num_layers = 1  # RNN层数

    model = RNN(input_size, hidden_size, output_size, num_layers)
    criterion = nn.CrossEntropyLoss()
    optimizer = optim.Adam(model.parameters(), lr=0.001)
    num_epochs = 10
    for epoch in range(num_epochs):
        for i, data in enumerate(train_loader):
            # 前向传播
            print(i)
            print(data)
            images, labels=data
            outputs = model(images)
            loss = criterion(outputs, labels)

            # 反向传播和优化
            optimizer.zero_grad()
            loss.backward()
            optimizer.step()

            if (i + 1) % 100 == 0:
                print("--")
if __name__ == '__main__':
    train()